--[[

Pallet Price Balancer for FS19

With Pallet Price Balancer the prices for palleted goods is no longer static and is now properly aligned towards the bulk price for each individual map. Also, the per ha/ac price for liquid fertilizer is now balanced towards the solid fertilizer.

Author:     w33zl
Version:    1.0
Modified:   2020-10-16

Changelog:
v1.0        Initial public release

]]

PalletBalancer = Mod:init()

function PalletBalancer:onBeforeStartMission()
    local SEED_PALLET_PRICE_FACTOR = 1.1
    local FERTILIZER_PALLET_PRICE_FACTOR = 1.2
    local LIQUID_FERTILIZER_PALLET_PRICE_FACTOR = 1.2
    local LIME_PALLET_PRICE_FACTOR = 1.1
    local BIGPACK_DISCOUNT_FACTOR = 0.9
    local DEFAULT_BIGBUY_THRESHOLD = 2000
    local LIME_BIGBUY_THRESHOLD = 2000
    local seedFillType = g_fillTypeManager:getFillTypeByName("SEEDS")
    local fertilizerFillType = g_fillTypeManager:getFillTypeByName("FERTILIZER")
    local liquidFertilizerFillType = g_fillTypeManager:getFillTypeByName("LIQUIDFERTILIZER")
    local limeFillType = g_fillTypeManager:getFillTypeByName("LIME")
    local seedBigBagFilename = "data/objects/bigBagContainer/bigBagContainerSeeds.xml"
    local seedPalletFilename = "data/objects/pallets/pioneer/pioneerPallet.xml"
    local fertilizerBigBagFilename = "data/objects/bigBagContainer/bigBagContainerFertilizer.xml"
    local limeBigBagFilename = "data/objects/bigBagContainer/bigBagContainerLime.xml"
    local largeSeedPallet = nil
    local largeFertilizerPallet = nil
    local smallLimePallet = nil
    local largeLimePallet = nil

    local function getSpec(storeItem, name)
        if storeItem ~= nil and storeItem.specs ~= nil then
            return storeItem.specs[name]
        else
            return nil
        end
    end

    local function getSpecFillTypeNames(storeItem)
        local specFillTypes = getSpec(storeItem, "fillTypes")

        if specFillTypes ~= nil then
            return specFillTypes.fillTypeNames
        else
            return ""
        end
    end

    local function setPalletPrice(storeItem, baseFillType, priceFactor, bigBuyDiscountThreshold)
        local capacitySpec = getSpec(storeItem, "capacity")
        if capacitySpec ~= nil and capacitySpec[1] ~= nil and capacitySpec[1][1] ~= nil and capacitySpec[1][1].capacity ~= nil then
            local fillCapacity = capacitySpec[1][1].capacity

            storeItem.price = baseFillType.pricePerLiter * fillCapacity * (priceFactor or 1.0)

            if PalletBalancer.debugMode == true then
                storeItem.name = storeItem.name .. " *"
            end

            PalletBalancer:printDebugVar("storeItem.name", storeItem.name)
            PalletBalancer:printDebugVar("storeItem.capacity", fillCapacity)

            if fillCapacity > (bigBuyDiscountThreshold or DEFAULT_BIGBUY_THRESHOLD) then
                storeItem.price = storeItem.price * BIGPACK_DISCOUNT_FACTOR
            end

            return fillCapacity
        end--if
    end    

    for _, storeItem in pairs(g_storeManager.items) do
        if storeItem.categoryName:upper() == "BIGBAGS" or storeItem.categoryName:upper() == "PALLETS" then

            if getSpecFillTypeNames(storeItem) == "seeds" then
                local palletCapacity = setPalletPrice(storeItem, seedFillType, SEED_PALLET_PRICE_FACTOR)

                if palletCapacity ~= nil and palletCapacity > DEFAULT_BIGBUY_THRESHOLD then
                    largeSeedPallet = largeSeedPallet or storeItem -- Add this pallet as large seed pallet (if not already set)
                end

                PalletBalancer:printDebugVar("storeItem[" .. storeItem.name .. "].specs.fillTypes.fillTypeNames", storeItem.specs.fillTypes.fillTypeNames)
            end

            if getSpecFillTypeNames(storeItem) == "fertilizer" then
                local palletCapacity = setPalletPrice(storeItem, fertilizerFillType, FERTILIZER_PALLET_PRICE_FACTOR)

                if palletCapacity ~= nil and palletCapacity > DEFAULT_BIGBUY_THRESHOLD then
                    largeFertilizerPallet = largeFertilizerPallet or storeItem -- Add this pallet as large fertilizer pallet (if not already set)
                end

                PalletBalancer:printDebugVar("storeItem[" .. storeItem.name .. "].specs.fillTypes.fillTypeNames", storeItem.specs.fillTypes.fillTypeNames)
            end

            if getSpecFillTypeNames(storeItem) == "liquidFertilizer" then
                setPalletPrice(storeItem, liquidFertilizerFillType, LIQUID_FERTILIZER_PALLET_PRICE_FACTOR)
            end

            if getSpecFillTypeNames(storeItem) == "lime" then
                local palletCapacity = setPalletPrice(storeItem, limeFillType, LIME_PALLET_PRICE_FACTOR, LIME_BIGBUY_THRESHOLD) or 0

                if palletCapacity == LIME_BIGBUY_THRESHOLD and storeItem.categoryName:upper() == "BIGBAGS" then
                    smallLimePallet = smallLimePallet or storeItem
                elseif palletCapacity >= LIME_BIGBUY_THRESHOLD and storeItem.categoryName:upper() == "PALLETS" then
                    largeLimePallet = largeLimePallet or storeItem -- Add this pallet as large lime pallet (if not already set)
                end

            end

        end

    end--for
end--function

FSBaseMission.onStartMission = Utils.prependedFunction(FSBaseMission.onStartMission, PalletBalancer.onBeforeStartMission, PalletBalancer);
